#ifndef __KLIBC_H__
#define __KLIBC_H__

#include <testos/types.h>

/* string.h functions */

void *memcpy(void *dst, const void *src, register unsigned int size ) ;
void *memset(void *dst, register int c, register unsigned int length ) ;
int memcmp(const void *s1, const void *s2, size_t n);

unsigned int strlen( register const char *str) ;
unsigned int strnlen(const char * s, size_t maxlen);

/**
 * @note Same as strncpy(), with a slightly different semantic.
 * Actually, strncpy(3C) says " The result will not be null-terminated
 * if the length of 'from' is n or more.".  Here, 'dst' is ALWAYS
 * null-terminated. And its total len will ALWAYS be <= len, with
 * null-terminating-char included.
 */
char *strzcpy( register char *dst, register const char *src,
               register int len ) ;

/**
 * @note Same as strncat(), with the same semantic : 'dst' is ALWAYS
 * null-terminated. And its total len will ALWAYS be <= len, with
 * null-terminating-char included.
 */
char *strzcat (char *dest, const char *src,
               const size_t len);
 
int strcmp(register const char *s1, register const char *s2 );
int strncmp(register const char *s1, register const char *s2,
	    register int len );

/* Basic stdarg.h macros. Taken from gcc support files */
#define __GNUC_VA_LIST 
typedef void *__gnuc_va_list;
typedef __gnuc_va_list va_list;
#define __va_rounded_size(TYPE) \
  (((sizeof (TYPE) + sizeof (int) - 1) / sizeof (int)) * sizeof (int))
#define va_start(AP, LASTARG) \
  (AP = ((__gnuc_va_list) __builtin_next_arg (LASTARG)))
#define va_end(AP) \
  ((void)0)
#define va_arg(AP, TYPE) \
  (AP = (__gnuc_va_list) ((char *) (AP) + __va_rounded_size (TYPE)),  \
   *((TYPE *) (void *) ((char *) (AP) - __va_rounded_size (TYPE))))
#define __va_copy(dest, src) \
  (dest) = (src)

/* stdarg.h functions. There might be a non-standard behavior: there
   will always be a trailing '\0' in the resulting string */
int vsnprintf(char *, size_t, const char *, va_list);
int snprintf(char *, size_t, const char *, /*args*/ ...)
  __attribute__ ((format (printf, 3, 4)));


/*
 * Pseudo-random generation functions. Useful to do some coverage
 * tests.
 */

/* Amplitude of the random number generation */
#define RAND_MAX 4294967291U

/* Pseudo-random number generation (MT unsafe) */
unsigned long int random (void);

/* Set random seed (MT unsafe) */
void srandom (unsigned long int seed);

#endif
